@@ -9,6 +9,10 @@ from io import IOBase
 from typing import Union, List, Optional, Dict, DefaultDict, Tuple, BinaryIO
 from urllib.parse import urlencode
 
+from requests.auth import HTTPBasicAuth, HTTPDigestAuth, AuthBase
+
+from .utils import get_real_file_path, triple_or_double_tostring
+
 try:
     import jstyleson as json
     from jsonschema import validate
@@ -20,7 +24,8 @@ from textx import TextXSyntaxError, metamodel_from_file
 
 from .dsl_jsonparser import json_or_array_to_json
 from .exceptions import *
-from .parse_models import Allhttp
+from .parse_models import Allhttp, AuthWrap, DigestAuth, BasicAuth, Line, Query, Http, NameWrap, UrlWrap, Header, \
+    MultiPartFile, FilesWrap, TripleOrDouble, Payload as ParsePayload
 from .property_schema import property_schema
 from .property_util import PropertyProvider
 
@@ -37,11 +42,7 @@ MIME_TYPE_JSON = "application/json"
 FORM_URLENCODED = "application/x-www-form-urlencoded"
 MULTIPART_FORM_INPUT = "multipart/form-data"
 
-if os.path.exists(__file__):
-    dir_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'http.tx')
-else:
-    dir_path = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'http.tx')
-dothttp_model = metamodel_from_file(dir_path)
+dothttp_model = metamodel_from_file(get_real_file_path(path="http.tx", current_file=__file__))
 
 
 def eprint(*args, **kwargs):
@@ -85,9 +86,10 @@ class HttpDef:
     url: str = None
     headers: dict = None
     query: dict = None
-    auth: Tuple[str] = None
+    auth: AuthBase = None
     payload: Optional[Payload] = None
     output: str = None
+    test_script: str = ""
 
     def get_har(self):
         target = {
@@ -134,6 +136,55 @@ class HttpDef:
     def get_headers(self):
         return [{"name": key, "value": value} for key, value in self.headers.items()] if self.headers else []
 
+    def get_http_from_req(self):
+        data = None
+        datajson = None
+        file = None
+        json_payload = None
+        fileswrap = None
+        type = None
+        if self.payload.filename:
+            file = self.payload.filename
+        elif isinstance(self.payload.data, str):
+            data = [TripleOrDouble(str=self.payload.data)]
+        elif isinstance(self.payload.data, dict):
+            datajson = None
+        elif self.payload.json:
+            json_payload = self.payload.json
+        elif self.payload.files:
+            fileswrap = FilesWrap([])
+            for filekey, multipartdata in self.payload.files:
+                fileswrap.files.append(
+                    MultiPartFile(name=filekey,
+                                  path=multipartdata[1].name if multipartdata[0] else multipartdata[1],
+                                  type=multipartdata[2] if len(multipartdata) > 2 else None)
+                )
+
+        payload = ParsePayload(data=data, datajson=datajson, file=file, json=json_payload, fileswrap=fileswrap,
+                               type=type)
+
+        query_lines = []
+        for key, values in self.query.items():
+            for value in values:
+                query_lines.append(Line(header=None, query=Query(key=key, value=value)))
+        auth_wrap = None
+        if self.auth:
+            if isinstance(self.auth, HTTPBasicAuth):
+                auth_wrap = AuthWrap(basic_auth=BasicAuth(self.auth.username, self.auth.password))
+            elif isinstance(self.auth, HTTPDigestAuth):
+                auth_wrap = AuthWrap(digest_auth=DigestAuth(self.auth.username, self.auth.password))
+        return Allhttp(allhttps=[Http(
+            namewrap=NameWrap(self.name),
+            urlwrap=UrlWrap(url=self.url, method=self.method),
+            lines=[
+                      Line(header=Header(key=key, value=value), query=None)
+                      for key, value in
+                      self.headers.items()] + query_lines
+            ,
+            payload=payload,
+            output=None, authwrap=auth_wrap
+        )])
+
 
 @dataclass
 class Property:
@@ -293,6 +344,16 @@ class BaseModelProcessor:
                                              value=target)
         else:
             self.http = self.model.allhttps[0]
+        self.base_http = None
+        if self.http.namewrap and self.http.namewrap.base:
+            base = self.http.namewrap.base
+            self.base_http = None
+            for http in self.model.allhttps:
+                if http.namewrap and http.namewrap.name == base:
+                    self.base_http = http
+                    break
+            if not self.base_http:
+                raise UndefinedHttpToExtend(target=self.http.namewrap.name, base=base)
 
     def validate_names(self):
         names = []
@@ -338,14 +399,20 @@ class HttpDefBase(BaseModelProcessor):
         """
         headers = {}
         headers.update(self.default_headers)
-        for line in self.http.lines:
+        self.load_headers_to_dict(self.base_http, headers)
+        self.load_headers_to_dict(self.http, headers)
+        request_logger.debug(
+            f'computed query params from `{self.file}` are `{headers}`')
+        self.httpdef.headers = headers
+
+    def load_headers_to_dict(self, http, headers):
+        if not http:
+            return
+        for line in http.lines:
             if header := line.header:
                 self.remove_quotes(header, "'")
                 self.remove_quotes(header, '"')
                 headers[self.get_updated_content(header.key)] = self.get_updated_content(header.value)
-        request_logger.debug(
-            f'computed query params from `{self.file}` are `{headers}`')
-        self.httpdef.headers = headers
 
     def load_url(self):
         request_logger.debug(
@@ -382,9 +449,8 @@ class HttpDefBase(BaseModelProcessor):
         # can be short circuted
         if not self.http.payload:
             return Payload()
-        elif self.http.payload.data or self.http.payload.multi:
-            data = self.http.payload.data or self.http.payload.multi[3:-3]
-            content = self.get_updated_content(data)
+        elif self.http.payload.data:
+            content = triple_or_double_tostring(self.http.payload.data, self.get_updated_content)
             mimetype = self.get_mimetype_from_buffer(content,
                                                      self.get_updated_content(self.http.payload.type))
             request_logger.debug(
@@ -464,9 +530,20 @@ class HttpDefBase(BaseModelProcessor):
             return sys.stdout
 
     def load_auth(self):
-        if auth_wrap := self.http.basic_auth_wrap:
-            self.httpdef.auth = self.get_updated_content(auth_wrap.username), self.get_updated_content(
-                auth_wrap.password)
+        auth_wrap = None
+        if self.http.authwrap:
+            auth_wrap = self.http.authwrap
+        elif self.base_http:
+            auth_wrap = self.base_http.authwrap
+        if auth_wrap:
+            if basic_auth := auth_wrap.basic_auth:
+                self.httpdef.auth = HTTPBasicAuth(self.get_updated_content(basic_auth.username),
+                                                  self.get_updated_content(
+                                                      basic_auth.password))
+            elif digest_auth := auth_wrap.digest_auth:
+                self.httpdef.auth = HTTPDigestAuth(self.get_updated_content(digest_auth.username),
+                                                   self.get_updated_content(
+                                                       digest_auth.password))
 
     def load_def(self):
         if self._loaded:
@@ -478,4 +555,11 @@ class HttpDefBase(BaseModelProcessor):
         self.load_query()
         self.load_payload()
         self.load_auth()
+        self.load_test_script()
         self._loaded = True
+
+    def load_test_script(self):
+        self.httpdef.test_script = ""
+        if script_wrap := self.http.script_wrap:
+            script = script_wrap.script[4:-2]
+            self.httpdef.test_script = script.strip()
