@@ -1,4 +1,4 @@
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import List, Optional
 
 
@@ -10,6 +10,7 @@ class Allhttp:
 @dataclass
 class NameWrap:
     name: str
+    base: Optional[str] = None
 
 
 @dataclass
@@ -24,6 +25,19 @@ class BasicAuth:
     password: str
 
 
+@dataclass
+class DigestAuth:
+    username: str
+    password: str
+
+
+@dataclass
+class AuthWrap:
+    # digest = DIGESTAUTH | basicauth = BASICAUTH
+    digest_auth: Optional[DigestAuth] = None
+    basic_auth: Optional[BasicAuth] = None
+
+
 @dataclass
 class Query:
     key: str
@@ -54,9 +68,15 @@ class FilesWrap:
     files: List[MultiPartFile]
 
 
+@dataclass
+class TripleOrDouble:
+    triple: Optional[str] = None
+    str: Optional[str] = None
+
+
 @dataclass
 class Payload:
-    data: Optional[str]
+    data: Optional[List[TripleOrDouble]]
     datajson: Optional
     file: Optional[str]
     json: Optional
@@ -70,14 +90,20 @@ class ToFile:
     output: str
 
 
+@dataclass
+class TestScript:
+    script: Optional[str]
+
+
 @dataclass
 class Http:
     namewrap: Optional[NameWrap]
     urlwrap: UrlWrap
-    basic_auth_wrap: Optional[BasicAuth]
+    authwrap: Optional[AuthWrap]
     lines: Optional[List[Line]]
     payload: Optional[Payload]
     output: Optional[ToFile]
+    script_wrap: Optional[TestScript] = field(default_factory=lambda: TestScript(''))
 
 
 @dataclass
